/////////////////////////////////////////////
//      Solar Position Algorithm (SPA)     //
//                   for                   //
//        Solar Radiation Application      //
//                                         //
//               May 12, 2003              //
//                                         //
//   Filename: SPA.C                       //
//                                         //
//   Afshin Michael Andreas                //
//   Afshin.Andreas@NREL.gov (303)384-6383 //
//                                         //
//   Measurement & Instrumentation Team    //
//   Solar Radiation Research Laboratory   //
//   National Renewable Energy Laboratory  //
//   1617 Cole Blvd, Golden, CO 80401      //
/////////////////////////////////////////////

/////////////////////////////////////////////
//   See the SPA.H header file for usage   //
//                                         //
//   This code is based on the NREL        //
//   technical report "Solar Position      //
//   Algorithm for Solar Radiation         //
//   Application" by I. Reda & A. Andreas  //
/////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
//
//   NOTICE
//   Copyright (C) 2008-2011 Alliance for Sustainable Energy, LLC, All Rights Reserved
//
//The Solar Position Algorithm ("Software") is code in development prepared by employees of the
//Alliance for Sustainable Energy, LLC, (hereinafter the "Contractor"), under Contract No.
//DE-AC36-08GO28308 ("Contract") with the U.S. Department of Energy (the "DOE"). The United
//States Government has been granted for itself and others acting on its behalf a paid-up, non-
//exclusive, irrevocable, worldwide license in the Software to reproduce, prepare derivative
//works, and perform publicly and display publicly. Beginning five (5) years after the date
//permission to assert copyright is obtained from the DOE, and subject to any subsequent five
//(5) year renewals, the United States Government is granted for itself and others acting on
//its behalf a paid-up, non-exclusive, irrevocable, worldwide license in the Software to
//reproduce, prepare derivative works, distribute copies to the public, perform publicly and
//display publicly, and to permit others to do so. If the Contractor ceases to make this
//computer software available, it may be obtained from DOE's Office of Scientific and Technical
//Information's Energy Science and Technology Software Center (ESTSC) at P.O. Box 1020, Oak
//Ridge, TN 37831-1020. THIS SOFTWARE IS PROVIDED BY THE CONTRACTOR "AS IS" AND ANY EXPRESS OR
//IMPLIED WARRANTIES, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
//AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE CONTRACTOR OR THE
//U.S. GOVERNMENT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER, INCLUDING BUT NOT LIMITED TO CLAIMS ASSOCIATED WITH THE LOSS OF DATA OR PROFITS,
//WHICH MAY RESULT FROM AN ACTION IN CONTRACT, NEGLIGENCE OR OTHER TORTIOUS CLAIM THAT ARISES
//OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
//
//The Software is being provided for internal, noncommercial purposes only and shall not be
//re-distributed. Please contact Jennifer Ramsey (Jennifer.Ramsey@nrel.gov) in the NREL
//Commercialization and Technology Transfer Office for information concerning a commercial
//license to use the Software.
//
//As a condition of using the Software in an application, the developer of the application
//agrees to reference the use of the Software and make this Notice readily accessible to any
//end-user in a Help|About screen or equivalent manner.
//
///////////////////////////////////////////////////////////////////////////////////////////////////

//This is a direct translation of the original c code into Delphi 7.
//Val gretchev March 2015.




unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Math, Menus, Mask;

type
  TForm1 = class(TForm)
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Edit6: TEdit;
    Edit7: TEdit;
    Edit8: TEdit;
    Edit9: TEdit;
    Edit10: TEdit;
    Edit11: TEdit;
    Edit12: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Edit13: TEdit;
    Edit14: TEdit;
    Edit15: TEdit;
    Edit16: TEdit;
    Edit17: TEdit;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Button1: TButton;
    StatusBar1: TStatusBar;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Label29: TLabel;
    Label30: TLabel;
    Label31: TLabel;
    Label32: TLabel;
    Label33: TLabel;
    Label34: TLabel;
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Clear1: TMenuItem;
    Input1: TMenuItem;
    Test1: TMenuItem;
    Whitby1: TMenuItem;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;

    procedure Button1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure Test1Click(Sender: TObject);
    procedure Whitby1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}




type
  PSpaData = ^TSpaData;

  TSpaData = record

    //------------------------INPUT VALUES--------------------------------------

    year: integer;
    month: integer;
    day: integer;
    hour: integer;
    minute: integer;
    second: integer;
    delta_ut1: double;
    delta_t: double;
    timezone: double;
    longitude: double;
    latitude: double;
    elevation: double;
    pressure: double;
    temperature: double;
    slope: double;
    azm_rotation: double;
    atmos_refract: double;
    switch: integer;


    //----------------Intermadiate OUTPUT VALUES--------------------------------

    jd: double;
    jc: double;

    jde: double;
    jce: double;
    jme: double;

    l: double;
    b: double;
    r: double;

    theta: double;
    beta: double;

    x0: double;
    x1: double;
    x2: double;
    x3: double;
    x4: double;

    del_psi: double;
    del_epsilon: double;
    epsilon0: double;
    epsilon: double;

    del_tau: double;
    lamda: double;
    nu0: double;
    nu: double;

    alpha: double;
    delta: double;

    h: double;
    xi: double;
    del_alpha: double;
    delta_prime: double;
    alpha_prime: double;
    h_prime: double;

    e0: double;
    del_e: double;
    e: double;

    eot: double;
    srha: double;
    ssha: double;
    sta: double;

    //----------------Final OUTPUT VALUES---------------------------------------

    zenith: double;
    azimuth_astro: double;
    azimuth: double;
    incidence: double;

    suntransit: double;
    sunrise: double;
    sunset: double;

  end;

type
  T1dArray = array[0..4] of double;
  X_ArrayPtr = ^T1dArray;


{
type

  TSer1 = (TERM_A, TERM_B, TERM_C, TERM_COUNT);
  TSer2 = (TERM_X0, TERM_X1, TERM_X2, TERM_X3, TERM_X4, TERM_X_COUNT);
  TSer3 = (TERM_PSI_A, TERM_PSI_B, TERM_EPS_C, TERM_EPS_D, TERM_PE_COUNT);
  TJD   = (JD_MINUS, JD_ZERO, JD_PLUS, JD_COUNT);
  TSun  = (SUN_TRANSIT, SUN_RISE, SUN_SET, SUN_COUNT);
}


const
  PI: double = 3.14159265358979;
  SUN_RADIUS: double = 0.26667;
  L_COUNT: integer = 6;
  B_COUNT: integer = 2;
  R_COUNT: integer = 5;
  Y_COUNT: integer = 63;
  L_MAX_SUBCOUNT: integer = 64;
  B_MAX_SUBCOUNT: integer = 5;
  R_MAX_SUBCOUNT: integer = 40;

  TERM_A: integer = 0;
  TERM_B: integer = 1;
  TERM_C: integer = 2;
  TERM_COUNT: integer = 3;

  TERM_X0: integer = 0;
  TERM_X1: integer = 1;
  TERM_X2: integer = 2;
  TERM_X3: integer = 3;
  TERM_X4: integer = 4;
  TERM_X_COUNT: integer = 5;

  TERM_PSI_A: integer = 0;
  TERM_PSI_B: integer = 1;
  TERM_EPS_C: integer = 2;
  TERM_EPS_D: integer = 3;
  TERM_PE_COUNT: integer = 4;

  JD_MINUS: integer = 0;
  JD_ZERO: integer = 1;
  JD_PLUS: integer = 2;
  JD_COUNT: integer = 3;

  SUN_TRANSIT: integer = 0;
  SUN_RISE: integer = 1;
  SUN_SET: integer = 2;
  SUN_COUNT: integer = 3;

  SPA_ZA: integer = 0;
  SPA_ZA_INC: integer = 1;
  SPA_ZA_RTS: integer = 2;
  SPA_ALL: integer = 3;

const
  l_subcount: array[0..5] of integer = (64,34,20,7,3,1);
  b_subcount: array[0..1] of integer = (5,2);
  r_subcount: array[0..4] of integer = (40,10,6,2,1);




type TL_TERMS = array[0..63,0..2] of double;
type TL_Arrays = array [0..5] of TL_TERMS;

const L_TERMS: TL_Arrays =
(
    (
        (175347046.0,0,0),
        (3341656.0,4.6692568,6283.07585),
        (34894.0,4.6261,12566.1517),
        (3497.0,2.7441,5753.3849),
        (3418.0,2.8289,3.5231),
        (3136.0,3.6277,77713.7715),
        (2676.0,4.4181,7860.4194),
        (2343.0,6.1352,3930.2097),
        (1324.0,0.7425,11506.7698),
        (1273.0,2.0371,529.691),
        (1199.0,1.1096,1577.3435),
        (990,5.233,5884.927),
        (902,2.045,26.298),
        (857,3.508,398.149),
        (780,1.179,5223.694),
        (753,2.533,5507.553),
        (505,4.583,18849.228),
        (492,4.205,775.523),
        (357,2.92,0.067),
        (317,5.849,11790.629),
        (284,1.899,796.298),
        (271,0.315,10977.079),
        (243,0.345,5486.778),
        (206,4.806,2544.314),
        (205,1.869,5573.143),
        (202,2.458,6069.777),
        (156,0.833,213.299),
        (132,3.411,2942.463),
        (126,1.083,20.775),
        (115,0.645,0.98),
        (103,0.636,4694.003),
        (102,0.976,15720.839),
        (102,4.267,7.114),
        (99,6.21,2146.17),
        (98,0.68,155.42),
        (86,5.98,161000.69),
        (85,1.3,6275.96),
        (85,3.67,71430.7),
        (80,1.81,17260.15),
        (79,3.04,12036.46),
        (75,1.76,5088.63),
        (74,3.5,3154.69),
        (74,4.68,801.82),
        (70,0.83,9437.76),
        (62,3.98,8827.39),
        (61,1.82,7084.9),
        (57,2.78,6286.6),
        (56,4.39,14143.5),
        (56,3.47,6279.55),
        (52,0.19,12139.55),
        (52,1.33,1748.02),
        (51,0.28,5856.48),
        (49,0.49,1194.45),
        (41,5.37,8429.24),
        (41,2.4,19651.05),
        (39,6.17,10447.39),
        (37,6.04,10213.29),
        (37,2.57,1059.38),
        (36,1.71,2352.87),
        (36,1.78,6812.77),
        (33,0.59,17789.85),
        (30,0.44,83996.85),
        (30,2.74,1349.87),
        (25,3.16,4690.48)
    ),
    (
        (628331966747.0,0,0),
        (206059.0,2.678235,6283.07585),
        (4303.0,2.6351,12566.1517),
        (425.0,1.59,3.523),
        (119.0,5.796,26.298),
        (109.0,2.966,1577.344),
        (93,2.59,18849.23),
        (72,1.14,529.69),
        (68,1.87,398.15),
        (67,4.41,5507.55),
        (59,2.89,5223.69),
        (56,2.17,155.42),
        (45,0.4,796.3),
        (36,0.47,775.52),
        (29,2.65,7.11),
        (21,5.34,0.98),
        (19,1.85,5486.78),
        (19,4.97,213.3),
        (17,2.99,6275.96),
        (16,0.03,2544.31),
        (16,1.43,2146.17),
        (15,1.21,10977.08),
        (12,2.83,1748.02),
        (12,3.26,5088.63),
        (12,5.27,1194.45),
        (12,2.08,4694),
        (11,0.77,553.57),
        (10,1.3,6286.6),
        (10,4.24,1349.87),
        (9,2.7,242.73),
        (9,5.64,951.72),
        (8,5.3,2352.87),
        (6,2.65,9437.76),
        (6,4.67,4690.48),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0)
    ),
    (
        (52919.0,0,0),
        (8720.0,1.0721,6283.0758),
        (309.0,0.867,12566.152),
        (27,0.05,3.52),
        (16,5.19,26.3),
        (16,3.68,155.42),
        (10,0.76,18849.23),
        (9,2.06,77713.77),
        (7,0.83,775.52),
        (5,4.66,1577.34),
        (4,1.03,7.11),
        (4,3.44,5573.14),
        (3,5.14,796.3),
        (3,6.05,5507.55),
        (3,1.19,242.73),
        (3,6.12,529.69),
        (3,0.31,398.15),
        (3,2.28,553.57),
        (2,4.38,5223.69),
        (2,3.75,0.98),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0)
    ),
    (
        (289.0,5.844,6283.076),
        (35,0,0),
        (17,5.49,12566.15),
        (3,5.2,155.42),
        (1,4.72,3.52),
        (1,5.3,18849.23),
        (1,5.97,242.73),
        (0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0)
    ),
    (
        (114.0,3.142,0),
        (8,4.13,6283.08),
        (1,3.84,12566.15),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0)
    ),
    (
        (1,3.14,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0)
    )
);


type TB_TERMS = array[0..4,0..2] of double;
type TB_Arrays = array [0..1] of TB_TERMS;

const B_TERMS: TB_Arrays =
(
    (
        (280.0,3.199,84334.662),
        (102.0,5.422,5507.553),
        (80,3.88,5223.69),
        (44,3.7,2352.87),
        (32,4,1577.34)
    ),
    (
        (9,3.9,5507.55),
        (6,1.73,5223.69),
        (0,0,0),(0,0,0),(0,0,0)
    )
);


type TR_TERMS = array[0..39,0..2] of double;
type TR_Arrays = array [0..4] of TR_TERMS;

const R_TERMS: TR_Arrays =
(
    (
        (100013989.0,0,0),
        (1670700.0,3.0984635,6283.07585),
        (13956.0,3.05525,12566.1517),
        (3084.0,5.1985,77713.7715),
        (1628.0,1.1739,5753.3849),
        (1576.0,2.8469,7860.4194),
        (925.0,5.453,11506.77),
        (542.0,4.564,3930.21),
        (472.0,3.661,5884.927),
        (346.0,0.964,5507.553),
        (329.0,5.9,5223.694),
        (307.0,0.299,5573.143),
        (243.0,4.273,11790.629),
        (212.0,5.847,1577.344),
        (186.0,5.022,10977.079),
        (175.0,3.012,18849.228),
        (110.0,5.055,5486.778),
        (98,0.89,6069.78),
        (86,5.69,15720.84),
        (86,1.27,161000.69),
        (65,0.27,17260.15),
        (63,0.92,529.69),
        (57,2.01,83996.85),
        (56,5.24,71430.7),
        (49,3.25,2544.31),
        (47,2.58,775.52),
        (45,5.54,9437.76),
        (43,6.01,6275.96),
        (39,5.36,4694),
        (38,2.39,8827.39),
        (37,0.83,19651.05),
        (37,4.9,12139.55),
        (36,1.67,12036.46),
        (35,1.84,2942.46),
        (33,0.24,7084.9),
        (32,0.18,5088.63),
        (32,1.78,398.15),
        (28,1.21,6286.6),
        (28,1.9,6279.55),
        (26,4.59,10447.39)
    ),
    (
        (103019.0,1.10749,6283.07585),
        (1721.0,1.0644,12566.1517),
        (702.0,3.142,0),
        (32,1.02,18849.23),
        (31,2.84,5507.55),
        (25,1.32,5223.69),
        (18,1.42,1577.34),
        (10,5.91,10977.08),
        (9,1.42,6275.96),
        (9,0.27,5486.78),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0)
    ),
    (
        (4359.0,5.7846,6283.0758),
        (124.0,5.579,12566.152),
        (12,3.14,0),
        (9,3.63,77713.77),
        (6,1.87,5573.14),
        (3,5.47,18849.23),
        (0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0)
    ),
    (
        (145.0,4.273,6283.076),
        (7,3.92,12566.15),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0)
    ),
    (
        (4,2.56,6283.08),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),
        (0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0),(0,0,0)
    )
);


const Y_TERMS: array [0..62,0..4] of double =
(
    (0,0,0,0,1),
    (-2,0,0,2,2),
    (0,0,0,2,2),
    (0,0,0,0,2),
    (0,1,0,0,0),
    (0,0,1,0,0),
    (-2,1,0,2,2),
    (0,0,0,2,1),
    (0,0,1,2,2),
    (-2,-1,0,2,2),
    (-2,0,1,0,0),
    (-2,0,0,2,1),
    (0,0,-1,2,2),
    (2,0,0,0,0),
    (0,0,1,0,1),
    (2,0,-1,2,2),
    (0,0,-1,0,1),
    (0,0,1,2,1),
    (-2,0,2,0,0),
    (0,0,-2,2,1),
    (2,0,0,2,2),
    (0,0,2,2,2),
    (0,0,2,0,0),
    (-2,0,1,2,2),
    (0,0,0,2,0),
    (-2,0,0,2,0),
    (0,0,-1,2,1),
    (0,2,0,0,0),
    (2,0,-1,0,1),
    (-2,2,0,2,2),
    (0,1,0,0,1),
    (-2,0,1,0,1),
    (0,-1,0,0,1),
    (0,0,2,-2,0),
    (2,0,-1,2,1),
    (2,0,1,2,2),
    (0,1,0,2,2),
    (-2,1,1,0,0),
    (0,-1,0,2,2),
    (2,0,0,2,1),
    (2,0,1,0,0),
    (-2,0,2,2,2),
    (-2,0,1,2,1),
    (2,0,-2,0,1),
    (2,0,0,0,1),
    (0,-1,1,0,0),
    (-2,-1,0,2,1),
    (-2,0,0,0,1),
    (0,0,2,2,1),
    (-2,0,2,0,1),
    (-2,1,0,2,1),
    (0,0,1,-2,0),
    (-1,0,1,0,0),
    (-2,1,0,0,0),
    (1,0,0,0,0),
    (0,0,1,2,0),
    (0,0,-2,2,2),
    (-1,-1,1,0,0),
    (0,1,1,0,0),
    (0,-1,1,2,2),
    (2,-1,-1,2,2),
    (0,0,3,2,2),
    (2,-1,0,2,2)
);


const PE_TERMS: array [0..62,0..3] of double =
(
    (-171996,-174.2,92025,8.9),
    (-13187,-1.6,5736,-3.1),
    (-2274,-0.2,977,-0.5),
    (2062,0.2,-895,0.5),
    (1426,-3.4,54,-0.1),
    (712,0.1,-7,0),
    (-517,1.2,224,-0.6),
    (-386,-0.4,200,0),
    (-301,0,129,-0.1),
    (217,-0.5,-95,0.3),
    (-158,0,0,0),
    (129,0.1,-70,0),
    (123,0,-53,0),
    (63,0,0,0),
    (63,0.1,-33,0),
    (-59,0,26,0),
    (-58,-0.1,32,0),
    (-51,0,27,0),
    (48,0,0,0),
    (46,0,-24,0),
    (-38,0,16,0),
    (-31,0,13,0),
    (29,0,0,0),
    (29,0,-12,0),
    (26,0,0,0),
    (-22,0,0,0),
    (21,0,-10,0),
    (17,-0.1,0,0),
    (16,0,-8,0),
    (-16,0.1,7,0),
    (-15,0,9,0),
    (-13,0,7,0),
    (-12,0,6,0),
    (11,0,0,0),
    (-10,0,5,0),
    (-8,0,3,0),
    (7,0,-3,0),
    (-7,0,0,0),
    (-7,0,3,0),
    (-7,0,3,0),
    (6,0,0,0),
    (6,0,-3,0),
    (6,0,-3,0),
    (-6,0,3,0),
    (-6,0,3,0),
    (5,0,0,0),
    (-5,0,3,0),
    (-5,0,3,0),
    (-5,0,3,0),
    (4,0,0,0),
    (4,0,0,0),
    (4,0,0,0),
    (-4,0,0,0),
    (-4,0,0,0),
    (-4,0,0,0),
    (3,0,0,0),
    (-3,0,0,0),
    (-3,0,0,0),
    (-3,0,0,0),
    (-3,0,0,0),
    (-3,0,0,0),
    (-3,0,0,0),
    (-3,0,0,0)
);








var
  mySpaRecord: TSpaData;
  mySpaRecordPtr: ^TSpaData;






{ In Math Unit
function DegToRad(const Degrees: Extended): Extended;
function RadToDeg(const Radians: Extended): Extended;
}



function Limit_Degrees(Degrees: double): double;
var
  Limited: double;
begin
  Degrees := Degrees / 360;
  Limited := 360 * (Degrees - Floor(Degrees));
  if (Limited < 0) then Limited := Limited + 360;
  Limit_Degrees := Limited;
end;

function Limit_Degrees180pm(Degrees: double): double;
var
  Limited: double;
begin
  Degrees := Degrees / 360;
  Limited := 360 * (Degrees - Floor(Degrees));
  if (Limited < -180) then Limited := Limited + 360
  else if (Limited > 180) then Limited := Limited - 360;
  Limit_Degrees180pm := Limited;
end;

function Limit_Degrees180(Degrees: double): double;
var
  Limited: double;
begin
  Degrees := Degrees / 180;
  Limited := 180 * (Degrees - Floor(Degrees));
  if (Limited < 0) then Limited := Limited + 180;
  Limit_Degrees180 := Limited;
end;

function Limit_Zero2One(Value: double): double;
var
  Limited: double;
begin
  Limited := Value - Floor(Value);
  if (Limited < 0) then Limited := Limited + 1;
  Limit_Zero2One := Limited;
end;

function Limit_Minutes(Minutes: double): double;
var
  Limited: double;
begin
  Limited := Minutes;
  if (Limited < -20) then Limited := Limited + 1440
  else if (Limited > 20) then Limited := Limited - 1440;
  Limit_Minutes := Limited;
end;

function DayFrac_To_Local_Hr(DayFrac: double; TimeZone: double): double;
begin
  DayFrac_To_Local_Hr := 24 * Limit_Zero2One(DayFrac + TimeZone / 24);
end;

function Third_Order_Polynomial(a: double; b: double; c: double; d: double; x: double): double;
begin
  Third_Order_Polynomial := ((a * x + b) * x + c) * x + d;
end;

{************************************************************************************************************}
function Validate_Inputs(SpaData: TSpaData): integer;
begin
  Validate_Inputs := 0;
end;


{************************************************************************************************************}


function Julian_Day(Year: integer; Month: integer; Day: integer; Hour: integer;
         Minute: integer; Second: double; Dut1: double; TZ: double): double;
var
  Day_Decimal: double;
  JulDay: double;
  a: double;
begin
    Day_Decimal := Day + (Hour - TZ + (Minute + (Second + Dut1) / 60) / 60) / 24;

    //Form1.Memo1.Lines.Add('Day_Decimal: '+FloatToStr(Day_Decimal));

    if (Month < 3) then
    begin
      Inc(Month, 12);
      Dec(Year);
    end;

    JulDay := Int(365.25 * (Year + 4716.0)) + Int(30.6001 * (Month + 1)) + Day_Decimal - 1524.5;

    //Form1.Memo1.Lines.Add('JulDay: '+FloatToStr(JulDay));

    a := 0;
    if (JulDay > 2299160) then
    begin
      a := Int(Year / 100);
      JulDay := JulDay + (2 - a + Int(a / 4));
    end;

    //Form1.Memo1.Lines.Add('a: '+FloatToStr(a));

    Julian_Day := JulDay;
end;

function Julian_Century(JD: double): double;
begin
  Julian_Century := (JD - 2451545) / 36525;
end;

function Julian_Ephemeris_Day(JD: double; Delta_t: double): double;
begin
  Julian_Ephemeris_Day := JD + Delta_t / 86400;
end;

function Julian_Ephemeris_Century(JDE: double): double;
begin
  Julian_Ephemeris_Century := (JDE - 2451545) / 36525;
end;

function Julian_Ephemeris_Millennium(JCE: double): double;
begin
  Julian_Ephemeris_Millennium := JCE / 10;
end;




{__________________________________________________________________________________________________}

function Earth_Periodic_Term_Summation_L(Terms: TL_TERMS; Count: integer; JME: double): double;
var
  i: integer;
  Sum: double;
begin
  Sum := 0;

  for i := 0 to Count-1 do
    Sum := Sum + (Terms[i][TERM_A] * cos(Terms[i][TERM_B] + Terms[i][TERM_C] * JME));

  //Form1.Memo1.Lines.Add('Sum_L: '+FloatToStr(Sum));

  Earth_Periodic_Term_Summation_L := Sum;
end;

function Earth_Periodic_Term_Summation_B(Terms: TB_TERMS; Count: integer; JME: double): double;
var
  i: integer;
  Sum: double;
begin
  Sum := 0;

  for i := 0 to Count-1 do
    Sum := Sum + (Terms[i][TERM_A] * cos(Terms[i][TERM_B] + Terms[i][TERM_C] * JME));

  //Form1.Memo1.Lines.Add('Sum: '+FloatToStr(Sum));

  Earth_Periodic_Term_Summation_B := Sum;
end;

function Earth_Periodic_Term_Summation_R(Terms: TR_TERMS; Count: integer; JME: double): double;
var
  i: integer;
  Sum: double;
begin
  Sum := 0;

  for i := 0 to Count-1 do
    Sum := Sum + (Terms[i][TERM_A] * cos(Terms[i][TERM_B] + Terms[i][TERM_C] * JME));

  //Form1.Memo1.Lines.Add('Sum: '+FloatToStr(Sum));

  Earth_Periodic_Term_Summation_R := Sum;
end;





function Earth_Values(Term_Sum: array of double; Count: integer; JME: double): double;
var
  i: integer;
  Sum: double;
begin
  Sum := 0;

  for i := 0 to Count-1 do
  begin
    Sum := Sum + ((Term_Sum[i] * Power(JME, i)));
  end;

  Sum := Sum / 1.0e8;

  Earth_Values := Sum;
end;

function Earth_Heliocentric_Longitude(JME: double): double;
var
  Sum: array[0..5] of double;  // L_COUNT
  i: integer;
begin
  for i := 0 to L_COUNT-1 do
  begin
    Sum[i] := Earth_Periodic_Term_Summation_L(L_TERMS[i], l_subcount[i], JME);
  end;
  Earth_Heliocentric_Longitude := Limit_Degrees(RadToDeg(Earth_Values(Sum, L_COUNT, JME)));
end;

function Earth_Heliocentric_Latitude(JME: double): double;
var
  Sum: array[0..1] of double;  // B_COUNT
  i: integer;
begin
  for i := 0 to B_COUNT-1 do
  begin
    Sum[i] := Earth_Periodic_Term_Summation_B(B_TERMS[i], b_subcount[i], JME);
  end;
  Earth_Heliocentric_Latitude := RadToDeg(Earth_Values(Sum, B_Count, JME));
end;

function Earth_Radius_Vector(JME: double): double;
var
  Sum: array[0..4] of double;  // R_COUNT
  i: integer;
begin
  for i := 0 to R_COUNT-1 do
  begin
    Sum[i] := Earth_Periodic_Term_Summation_R(R_TERMS[i], r_subcount[i], JME);
  end;
  Earth_Radius_Vector := Earth_Values(Sum, R_Count, JME);
end;


{_______________________________________________________________________________________________________________}


function Geocentric_Longitude(L: double): double;
var
  Theta: double;
begin
  Theta := L + 180;

  if (Theta >= 360) then Theta := Theta - 360;
  Geocentric_Longitude := Theta;
end;

function Geocentric_Latitude(B: double): double;
begin
  Geocentric_Latitude := -B;
end;

function Mean_Elongation_Moon_Sun(JCE: double): double;
begin
  Mean_Elongation_Moon_Sun := Third_Order_Polynomial(-1/300000, -0.0001603, 35999.05034, 357.52772, jce);
end;

function Mean_Anomaly_Sun(JCE: double): double;
begin
  Mean_Anomaly_Sun := Third_Order_Polynomial(-1/300000, -0.0001603, 35999.05034, 357.52772, jce);
end;

function Mean_Anomaly_Moon(JCE: double): double;
begin
  Mean_Anomaly_Moon := Third_Order_Polynomial(1/56250, 0.0086972, 477198.867398, 134.96298, jce);
end;

function Argument_Latitude_Moon(JCE: double): double;
begin
  Argument_Latitude_Moon := Third_Order_Polynomial(1/327270, -0.0036825, 483202.017538, 93.27191, jce);
end;

function Ascending_Longitude_Moon(JCE: double): double;
begin
  Ascending_Longitude_Moon := Third_Order_Polynomial(1/450000, 0.0020708, -1934.136261, 125.04452, jce);
end;

function xy_term_summation(i: integer; x: T1dArray): double;
var
  j: integer;
  sum: double;
begin
  sum := 0;
  for j := 0 to Length(x) do
    sum := sum + (x[j] * Y_TERMS[i][j]);

  xy_term_summation := sum;
end;




procedure Nutation_Longitude_And_Obliquity(JCE: double; x: T1dArray; var P1: double; var P2: double);
var
  i: integer;
  xy_term_sum: double;
  sum_psi: double;
  sum_epsilon: double;
begin
  sum_psi := 0;
  sum_epsilon := 0;
  for i := 0 to Y_COUNT-1 do
  begin
    xy_term_sum := DegToRad(xy_term_summation(i,x));
    sum_psi := sum_psi + (PE_TERMS[i][TERM_PSI_A] + jce * PE_TERMS[i][TERM_PSI_B]) * sin(xy_term_sum);
    sum_epsilon := sum_epsilon + (PE_TERMS[i][TERM_EPS_C] + jce * PE_TERMS[i][TERM_EPS_D]) * cos(xy_term_sum);
  end;

  P1 := sum_psi / 36000000;
  P2 := sum_epsilon / 36000000;
end;

function Ecliptic_Mean_Obliquity(JME: double): double;
var
  u: double;
begin
  u := JME/10;
  Ecliptic_Mean_Obliquity := 84381.448 + u * (-4680.93 + u * (-1.55 + u * (1999.25 + u * (-51.38 + u * (-249.67 +
                             u * (-39.05 + u * (7.12 + u * (27.87 + u * (5.79 + u * 2.45)))))))));
end;

function Ecliptic_True_Obliquity(delta_epsilon: double; epsilon0: double): double;
begin
  Ecliptic_True_Obliquity := delta_epsilon + epsilon0 / 3600.0;
end;

function Aberration_Correction(r: double): double;
begin
  Aberration_Correction := -20.4898 / (3600.0 * r);
end;

function Apparent_Sun_Longitude(theta: double; delta_psi: double; delta_tau: double): double;
begin
  Apparent_Sun_Longitude := theta + delta_psi + delta_tau;
end;

function Greenwich_Mean_Sidereal_Time(jd: double; jc: double): double;
begin
  //Form1.Memo1.Lines.Add('jd: '+FloatToStr(jd));

  Greenwich_Mean_Sidereal_Time := Limit_Degrees(280.46061837 + 360.98564736629 *
            (jd - 2451545.0) + jc * jc * (0.000387933 - jc / 38710000.0));
end;

function Greenwich_Sidereal_Time(nu0: double; delta_psi: double; epsilon: double): double;
begin
  //Form1.Memo1.Lines.Add('epsilon: '+FloatToStr(epsilon));


  Greenwich_Sidereal_Time := nu0 + delta_psi * cos(DegToRad(epsilon));
end;

function Geocentric_Right_Ascension(lamda: double; epsilon: double; beta: double): double;
var
  lamda_rad: double;
  epsilon_rad: double;
  Temp1: double;




begin
  lamda_rad := DegToRad(lamda);
  epsilon_rad := DegToRad(epsilon);


  Temp1 := Limit_Degrees(RadToDeg(ArcTan2(sin(lamda_rad) * cos(epsilon_rad) - tan(DegToRad(beta)) * sin(epsilon_rad), cos(lamda_rad))));
{
  Form1.Memo1.Lines.Add('lamda: '+FloatToStr(lamda));
  Form1.Memo1.Lines.Add('epsilon: '+FloatToStr(epsilon));
  Form1.Memo1.Lines.Add('beta: '+FloatToStr(beta));

  Form1.Memo1.Lines.Add('Temp1: '+FloatToStr(Temp1));
}


  Geocentric_Right_Ascension := Limit_Degrees(RadToDeg(ArcTan2(sin(lamda_rad) *
  cos(epsilon_rad) - tan(DegToRad(beta)) * sin(epsilon_rad), cos(lamda_rad))));
end;

function Geocentric_Declination(beta: double; epsilon: double; lamda: double): double;
var
  beta_rad: double;
  epsilon_rad: double;
begin
  beta_rad := DegToRad(beta);
  epsilon_rad := DegToRad(epsilon);

  Geocentric_Declination := RadToDeg(ArcSin(sin(beta_rad) * cos(epsilon_rad) +
                        cos(beta_rad) * sin(epsilon_rad) * sin(DegToRad(lamda))));
end;

function Observer_Hour_Angle(nu: double; longitude: double; alpha_deg: double): double;
begin

{
  Form1.Memo1.Lines.Add('nu: '+FloatToStr(nu));
  Form1.Memo1.Lines.Add('longitude: '+FloatToStr(longitude));
  Form1.Memo1.Lines.Add('alpha_deg: '+FloatToStr(alpha_deg));
  Form1.Memo1.Lines.Add('Arithmetic result: '+FloatToStr(nu + longitude - alpha_deg));
}

  Observer_Hour_Angle := Limit_Degrees(nu + longitude - alpha_deg);
end;

function Sun_Equatorial_Horizontal_Parallax(r: double): double;
begin
  Sun_Equatorial_Horizontal_Parallax := 8.794 / (3600.0 * r);
end;

procedure Right_Ascension_Parallax_And_Topocentric_Dec(latitude: double; elevation: double;
          xi: double; h: double; delta: double; var delta_alpha: double; var delta_prime: double);
var
  delta_alpha_rad: double;
  lat_rad: double;
  xi_rad: double;
  h_rad: double;
  delta_rad: double;
  u: double;
  y: double;
  x: double;
begin
  lat_rad := DegToRad(latitude);
  xi_rad := DegToRad(xi);
  h_rad := DegToRad(h);
  delta_rad := DegToRad(delta);
  u := ArcTan(0.99664719 * Tan(lat_rad));
  y := 0.99664719 * sin(u) + elevation * sin(lat_rad) / 6378140.0;
  x := cos(u) + elevation * cos(lat_rad) / 6378140.0;

  delta_alpha_rad := ArcTan2(- x * sin(xi_rad) * sin(h_rad),
                      cos(delta_rad) - x * sin(xi_rad) * cos(h_rad));

  delta_prime := RadToDeg(ArcTan2((sin(delta_rad) - y * sin(xi_rad)) * cos(delta_alpha_rad),
                      cos(delta_rad) - x * sin(xi_rad) * cos(h_rad)));

  delta_alpha := RadToDeg(delta_alpha_rad);
end;

function Topocentric_Right_Ascension(alpha_deg: double; delta_alpha: double): double;
begin
  Topocentric_Right_Ascension := alpha_deg + delta_alpha;
end;

function Topocentric_Local_Hour_Angle(h: double; delta_alpha: double): double;
begin
  Topocentric_Local_Hour_Angle := h - delta_alpha;
end;

function Topocentric_Elevation_Angle(latitude: double; delta_prime: double; h_prime: double): double;
var
  lat_rad: double;
  delta_prime_rad: double;
begin
  lat_rad := DegToRad(latitude);
  delta_prime_rad := DegToRad(delta_prime);

  Topocentric_Elevation_Angle := RadToDeg(ArcSin(sin(lat_rad) * sin(delta_prime_rad) +
                        cos(lat_rad) * cos(delta_prime_rad) * cos(DegToRad(h_prime))));
end;

function Atmospheric_Refraction_Correction(pressure: double; temperature: double;
                      atmos_refract: double; e0: double): double;
var
  del_e: double;
begin
  del_e := 0;

  if (e0 >= -1 * (SUN_RADIUS + atmos_refract)) then
      del_e := (pressure / 1010.0) * (283.0 / (273.0 + temperature)) *
                1.02 / (60.0 * tan(DegToRad(e0 + 10.3 / (e0 + 5.11))));

  Atmospheric_Refraction_Correction := del_e;
end;

function Topocentric_Elevation_Angle_Corrected(e0: double; delta_e: double): double;
begin
  Topocentric_Elevation_Angle_Corrected := e0 + delta_e;
end;

function Topocentric_Zenith_Angle(e: double): double;
begin
  Topocentric_Zenith_Angle := 90.0 - e;
end;

function Topocentric_Azimuth_Angle_Astro(h_prime: double; latitude: double; delta_prime: double): double;
var
  h_prime_rad: double;
  lat_rad: double;
  Temp1: double;
  Temp2: double;
  Temp3: double;
  Temp4: double;
  Temp5: double;
begin
  h_prime_rad := DegToRad(h_prime);
  lat_rad := DegToRad(latitude);
{
  Form1.Memo1.Lines.Add('h_prime: '+FloatToStr(h_prime));
  Form1.Memo1.Lines.Add('latitude: '+FloatToStr(latitude));
  Form1.Memo1.Lines.Add('delta_prime: '+FloatToStr(delta_prime));
  Temp1 := sin(h_prime_rad);
  Temp2 := cos(h_prime_rad) * sin(lat_rad) - tan(DegToRad(delta_prime)) * cos(lat_rad);
  Temp3 := ArcTan2(Temp1,Temp2);
  Form1.Memo1.Lines.Add('Temp3: '+FloatToStr(Temp3));
  Temp4 := RadToDeg(Temp3);
  Form1.Memo1.Lines.Add('Temp4: '+FloatToStr(Temp4));
  Temp5 := Limit_Degrees(Temp4);
  Form1.Memo1.Lines.Add('Temp5: '+FloatToStr(Temp5));
}
  Topocentric_Azimuth_Angle_Astro := Limit_Degrees(RadToDeg(ArcTan2(sin(h_prime_rad), cos(h_prime_rad) * sin(lat_rad) - tan(DegToRad(delta_prime)) * cos(lat_rad))));
end;

function Topocentric_Azimuth_Angle(azimuth_astro: double): double;
begin
  Topocentric_Azimuth_Angle := Limit_Degrees(azimuth_astro + 180.0);
end;

function Surface_Incidence_Angle(zenith: double; azimuth_astro: double;
                   azm_rotation: double; slope: double): double;
var
  zenith_rad: double;
  slope_rad: double;
begin
  zenith_rad := DegToRad(zenith);
  slope_rad := DegToRad(slope);

  Surface_Incidence_Angle := RadToDeg(ArcCos(cos(zenith_rad) * cos(slope_rad)  +
  sin(slope_rad ) * sin(zenith_rad) * cos(DegToRad(azimuth_astro - azm_rotation))));
end;

function Sun_Mean_Longitude(jme: double): double;
begin
  Sun_Mean_Longitude := Limit_Degrees(280.4664567 + jme * (360007.6982779 +
    jme * (0.03032028 + jme * (1.0/49931.0   + jme * (-1.0/15300.0 + jme *
    (-1.0/2000000.0))))));
end;

function Eot(m: double; alpha: double; del_psi: double; epsilon: double): double;
begin
  Eot := Limit_Minutes(4.0 * (m - 0.0057183 - alpha + del_psi * cos(DegToRad(epsilon))));
end;

function Approx_Sun_Transit_Time(alpha_zero: double; longitude: double; nu: double): double;
begin
  Approx_Sun_Transit_Time := (alpha_zero - longitude - nu) / 360.0;
end;

function Sun_Hour_Angle_At_Rise_Set(latitude: double; delta_zero: double; h0_prime: double): double;
var
  h0: double;
  latitude_rad: double;
  delta_zero_rad: double;
  argument: double;
begin
  h0 := -99999.0;
  latitude_rad := DegToRad(latitude);
  delta_zero_rad := DegToRad(delta_zero);
  argument := (sin(DegToRad(h0_prime)) - sin(latitude_rad) * sin(delta_zero_rad)) /
              (cos(latitude_rad) * cos(delta_zero_rad));

  if (Abs(argument) <= 1) then h0 := Limit_Degrees180(RadToDeg(ArcCos(argument)));

  Sun_Hour_Angle_At_Rise_Set := h0;
end;

procedure Approx_Sun_Rise_And_Set(var m_rts: array of double; h0: double);
var
  h0_dfrac: double;
begin
  h0_dfrac := h0 / 360.0;

  m_rts[SUN_RISE]    := Limit_Zero2One(m_rts[SUN_TRANSIT] - h0_dfrac);
  m_rts[SUN_SET]     := limit_Zero2One(m_rts[SUN_TRANSIT] + h0_dfrac);
  m_rts[SUN_TRANSIT] := Limit_Zero2One(m_rts[SUN_TRANSIT]);
end;

function Rts_Alpha_Delta_Prime(var ad: array of double; n: double): double;
var
  a: double;
  b: double;
begin
  a := ad[JD_ZERO] - ad[JD_MINUS];
  b := ad[JD_PLUS] - ad[JD_ZERO];

  if (Abs(a) >= 2.0) then a := Limit_Zero2One(a);
  if (Abs(b) >= 2.0) then b := Limit_Zero2One(b);

  Rts_Alpha_Delta_Prime := ad[JD_ZERO] + n * (a + b + (b-a) * n) / 2.0;
end;

function Rts_Sun_Altitude(latitude: double; delta_prime: double; h_prime: double): double;
var
  latitude_rad: double;
  delta_prime_rad: double;
begin
  latitude_rad := DegToRad(latitude);
  delta_prime_rad := DegToRad(delta_prime);

  Rts_Sun_Altitude := RadToDeg(ArcSin(sin(latitude_rad) * sin(delta_prime_rad) +
            cos(latitude_rad) * cos(delta_prime_rad) * cos(DegToRad(h_prime))));
end;

function Sun_Rise_And_Set(var m_rts: array of double; var h_rts: array of double; var delta_prime: array of double;
    latitude: double; var h_prime: array of double; h0_prime: double; sun: integer): double;
begin
  Sun_Rise_And_Set := m_rts[sun] + (h_rts[sun] - h0_prime) / (360.0 *
  cos(DegToRad(delta_prime[sun])) * cos(DegToRad(latitude)) * sin(DegToRad(h_prime[sun])));
end;




procedure Calculate_Geocentric_Sun_Right_Ascension_And_Declination(var SpaData: TSpaData);
var
  x: T1dArray;


begin
  SpaData.jc := Julian_Century(SpaData.jd);

  SpaData.jde := Julian_Ephemeris_Day(SpaData.jd, SpaData.delta_t);
  SpaData.jce := Julian_Ephemeris_Century(SpaData.jde);
  SpaData.jme := Julian_Ephemeris_Millennium(SpaData.jce);

  SpaData.l := Earth_Heliocentric_Longitude(SpaData.jme);
  SpaData.b := Earth_Heliocentric_Latitude(SpaData.jme);
  SpaData.r := Earth_Radius_Vector(SpaData.jme);

  SpaData.theta := Geocentric_Longitude(SpaData.l);
  SpaData.beta := Geocentric_Latitude(SpaData.b);

  x[0] := Mean_Elongation_Moon_Sun(SpaData.jce);  SpaData.x0 := x[0];
  x[1] := Mean_Anomaly_Sun(SpaData.jce);  SpaData.x1 := x[1];
  x[2] := Mean_Anomaly_Moon(SpaData.jce);  SpaData.x2 := x[2];
  x[3] := Argument_Latitude_Moon(SpaData.jce);  SpaData.x3 := x[3];
  x[4] := Ascending_Longitude_Moon(SpaData.jce);  SpaData.x4 := x[4];

  Nutation_Longitude_And_Obliquity(SpaData.jce, x, SpaData.del_psi, SpaData.del_epsilon);

  SpaData.epsilon0 := Ecliptic_Mean_Obliquity(SpaData.jme);
  SpaData.epsilon :=  Ecliptic_True_Obliquity(SpaData.del_epsilon, SpaData.epsilon0);

  SpaData.del_tau := Aberration_Correction(SpaData.r);
  SpaData.lamda := Apparent_Sun_Longitude(SpaData.theta, SpaData.del_psi, SpaData.del_tau);
  SpaData.nu0 := Greenwich_Mean_Sidereal_Time (SpaData.jd, SpaData.jc);
  SpaData.nu := Greenwich_Sidereal_Time (SpaData.nu0, SpaData.del_psi, SpaData.epsilon);

  SpaData.alpha := Geocentric_Right_Ascension(SpaData.lamda, SpaData.epsilon, SpaData.beta);

  SpaData.delta := Geocentric_Declination(SpaData.beta, SpaData.epsilon, SpaData.lamda);

end;

procedure Calculate_Eot_And_Sun_Rise_Transit_Set(var SpaData: TSpaData);
var
  sun_rts: TSpaData;

  nu, m, h0, n: double;
  alpha: array[0..2] of double; // JD_COUNT = 3
  delta: array[0..2] of double; // JD_COUNT = 3
  m_rts: array[0..2] of double; // SUN_COUNT = 3
  nu_rts: array[0..2] of double;  // Sun_COUNT = 3
  h_rts: array[0..2] of double; // SUN_COUNT = 3
  alpha_prime: array[0..2] of double; // SUN_COUNT = 3
  delta_prime: array[0..2] of double; // SUN_COUNT = 3
  h_prime: array[0..2] of double; // SUN_COUNT = 3

  h0_prime: double;
  i: integer;
begin
  h0_prime := -1 * (SUN_RADIUS + SpaData.atmos_refract);

  sun_rts := SpaData;

  m := Sun_Mean_Longitude(SpaData.jme);

  SpaData.eot := Eot(m, SpaData.alpha, SpaData.del_psi, SpaData.epsilon);

  sun_rts.hour := 0;
  sun_rts.minute := 0;
  sun_rts.second := 0;
  sun_rts.delta_ut1 := 0;
  sun_rts.timezone := 0;

  sun_rts.jd := Julian_Day (sun_rts.year, sun_rts.month, sun_rts.day, sun_rts.hour,
                             sun_rts.minute, sun_rts.second, sun_rts.delta_ut1, sun_rts.timezone);

  Calculate_Geocentric_Sun_Right_Ascension_And_Declination(sun_rts);
  nu := sun_rts.nu;

  sun_rts.delta_t := 0;
  sun_rts.jd := sun_rts.jd - 1;

  for i := 0 to JD_COUNT-1 do
  begin
    Calculate_Geocentric_Sun_Right_Ascension_And_Declination(sun_rts);
    alpha[i] := sun_rts.alpha;
    delta[i] := sun_rts.delta;
    sun_rts.jd := sun_rts.jd + 1;
  end;

  m_rts[SUN_TRANSIT] := Approx_Sun_Transit_Time(alpha[JD_ZERO], SpaData.longitude, nu);
  h0 := Sun_Hour_Angle_At_Rise_Set(SpaData.latitude, delta[JD_ZERO], h0_prime);

  if (h0 >= 0) then
  begin
    Approx_Sun_Rise_And_Set(m_rts, h0);
    for i := 0 to SUN_COUNT-1 do
    begin
      nu_rts[i] := nu + 360.985647 * m_rts[i];
      n := m_rts[i] + SpaData.delta_t / 86400.0;
      alpha_prime[i] := Rts_Alpha_Delta_Prime(alpha, n);
      delta_prime[i] := Rts_Alpha_Delta_Prime(delta, n);
      h_prime[i] := Limit_Degrees180pm(nu_rts[i] + SpaData.longitude - alpha_prime[i]);
      h_rts[i] := Rts_Sun_Altitude(SpaData.latitude, delta_prime[i], h_prime[i]);
    end;
    SpaData.srha := h_prime[SUN_RISE];
    SpaData.ssha := h_prime[SUN_SET];
    SpaData.sta := h_rts[SUN_TRANSIT];

    SpaData.suntransit := DayFrac_To_Local_Hr(m_rts[SUN_TRANSIT] - h_prime[SUN_TRANSIT] / 360.0, SpaData.timezone);
    SpaData.sunrise := DayFrac_To_Local_Hr(Sun_Rise_And_Set(m_rts, h_rts, delta_prime, SpaData.latitude, h_prime, h0_prime, SUN_RISE), SpaData.timezone);
    SpaData.sunset := DayFrac_To_Local_Hr(Sun_Rise_And_Set(m_rts, h_rts, delta_prime, SpaData.latitude, h_prime, h0_prime, SUN_SET),  SpaData.timezone);
  end
  else
  begin
    SpaData.srha := -99999.0;
    SpaData.ssha := -99999.0;
    SpaData.sta := -99999.0;
    SpaData.suntransit := -99999.0;
    SpaData.sunrise := -99999.0;
    SpaData.sunset := -99999.0;
  end;
end;



function Spa_Calculate(var SpaData: TSpaData): integer;
var
  return_result: integer;
begin
  return_result := Validate_Inputs(SpaData);

  if (return_result = 0) then
  begin
    SpaData.jd := Julian_Day(SpaData.year, SpaData.month, SpaData.day, SpaData.hour,
           SpaData.minute, SpaData.second, SpaData.delta_ut1, SpaData.timezone);

    //Form1.Memo1.Lines.Add('jd: '+FloatToStr(SpaData.jd));




    Calculate_Geocentric_Sun_Right_Ascension_And_Declination(SpaData);

    SpaData.h := Observer_Hour_angle(SpaData.nu, SpaData.longitude, SpaData.alpha);
    //Form1.Memo1.Lines.Add('h: '+FloatToStr(SpaData.h));



    SpaData.xi := Sun_Equatorial_Horizontal_Parallax(SpaData.r);

    Right_Ascension_Parallax_And_Topocentric_Dec(SpaData.latitude, SpaData.elevation, SpaData.xi,
                                SpaData.h, SpaData.delta, SpaData.del_alpha, SpaData.delta_prime);
    SpaData.alpha_prime := Topocentric_Right_Ascension(SpaData.alpha, SpaData.del_alpha);
    SpaData.h_prime := Topocentric_Local_Hour_Angle(SpaData.h, SpaData.del_alpha);

    SpaData.e0 := Topocentric_Elevation_Angle(SpaData.latitude, SpaData.delta_prime, SpaData.h_prime);
    SpaData.del_e := Atmospheric_Refraction_Correction(SpaData.pressure, SpaData.temperature, SpaData.atmos_refract, SpaData.e0);

    SpaData.e := Topocentric_Elevation_Angle_Corrected(SpaData.e0, SpaData.del_e);

    SpaData.zenith := Topocentric_Zenith_Angle(SpaData.e);

    SpaData.azimuth_astro := Topocentric_Azimuth_Angle_Astro(SpaData.h_prime, SpaData.latitude, SpaData.delta_prime);

    SpaData.azimuth := Topocentric_Azimuth_Angle(SpaData.azimuth_astro);

    if ((SpaData.switch = SPA_ZA_INC) or (SpaData.switch = SPA_ALL)) then
            SpaData.incidence  := Surface_Incidence_Angle(SpaData.zenith, SpaData.azimuth_astro, SpaData.azm_rotation, SpaData.slope);

    if ((SpaData.switch = SPA_ZA_RTS) or (SpaData.switch = SPA_ALL)) then
            Calculate_Eot_And_Sun_Rise_Transit_Set(SpaData);
  end;

  Spa_Calculate := result;

end;



//******************************************************************************
//  Helper Functions
//******************************************************************************

function GetFirstToken(S: string; Token: Char): string;
var
  Count: Integer;
  Temp: string;
begin
  Count := Pos(Token, S);
  if Count < 1 then
  begin
    GetFirstToken := '';
    Exit;
  end;
  Dec(Count);
  Temp := Copy(S,1,Count);
  GetFirstToken := Temp;
end;

function StripFirstToken(S: string; Token: Char): string;
var
  Count: integer;
  Temp: string;
begin
  Count := Pos(Token, S);
  if Count = 0 then begin
    StripFirstToken := S;
    Exit;
  end;
  Temp := S;
  Delete(Temp,1,Count);
  StripFirstToken := Temp;
end;

function CleanString(S: string): string;
var
  Temp: string;
  i: integer;
  Ch: char;

begin

  for i := 1 to Length(S) do
  begin
    Ch := S[i];
    if Ch > chr(31) then
    begin
      Temp := Temp + Ch;
    end;
  end;
  CleanString := Temp;
end;

function Int2StrPad0(N: LongInt; Len: Integer): String;
var
  S : string;
begin
  Str(N:0,S);
  while Length(S) < Len do
    S := '0' + S;
  Int2StrPad0 := S;
end;

//******************************************************************************
//   Program Control
//******************************************************************************

procedure TForm1.Button1Click(Sender: TObject);
var
  hrs: integer;
  min: integer;
  sec: integer;
  a: double;
begin
  try
    mySpaRecord.year := StrToInt(Edit1.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.month := StrToInt(Edit2.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.day := StrToInt(Edit3.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.hour := StrToInt(Edit4.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.minute := StrToInt(Edit5.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.second := StrToInt(Edit6.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.delta_ut1 := StrToFloat(Edit7.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.delta_t := StrToFloat(Edit8.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.timezone := StrToFloat(Edit9.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.longitude := StrToFloat(Edit10.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.latitude := StrToFloat(Edit11.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.elevation := StrToFloat(Edit12.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.pressure := StrToFloat(Edit13.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.temperature := StrToFloat(Edit14.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.slope := StrToFloat(Edit15.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.azm_rotation := StrToFloat(Edit16.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  try
    mySpaRecord.atmos_refract := StrToFloat(Edit17.Text);
  except
    on Exception: EConvertError do
       ShowMessage(Exception.Message);
  end;

  mySpaRecord.switch := SPA_ALL;

  mySpaRecordPtr := @mySpaRecord;

  Spa_Calculate(mySpaRecord);

  Memo1.Lines.Add('');
  Memo1.Lines.Add('Julian Day:    '+FloatToStr(mySpaRecord.jd));
  Memo1.Lines.Add('L:             '+FloatToStr(mySpaRecord.l)+' degrees');
  Memo1.Lines.Add('B:             '+FloatToStr(mySpaRecord.b)+' degrees');
  Memo1.Lines.Add('R:             '+FloatToStr(mySpaRecord.r)+' AU');
  Memo1.Lines.Add('H:             '+FloatToStr(mySpaRecord.h)+' degrees');
  Memo1.Lines.Add('Delta Psi:     '+FloatToStr(mySpaRecord.del_psi)+' degrees');
  Memo1.Lines.Add('Delta Epsilon: '+FloatToStr(mySpaRecord.del_epsilon)+' degrees');
  Memo1.Lines.Add('Epsilon:       '+FloatToStr(mySpaRecord.epsilon)+' degrees');
  //Memo1.Lines.Add('Zenith:        '+FloatToStr(mySpaRecord.zenith)+' degrees');
  //Memo1.Lines.Add('Azimuth:       '+FloatToStr(mySpaRecord.azimuth)+' degrees');
  //Memo1.Lines.Add('Incidence:     '+FloatToStr(mySpaRecord.incidence)+' degrees');

  Memo1.Lines.Add('');
  Memo1.Lines.Add('Zenith:        '+FloatToStr(mySpaRecord.zenith)+' degrees');
  Memo1.Lines.Add('Azimuth astro: '+FloatToStr(mySpaRecord.azimuth_astro)+' degrees');
  Memo1.Lines.Add('Azimuth:       '+FloatToStr(mySpaRecord.azimuth)+' degrees');
  Memo1.Lines.Add('Incidence:     '+FloatToStr(mySpaRecord.incidence)+' degrees');

  Memo1.Lines.Add('');
  hrs := Trunc(mySpaRecord.sunrise);
  min := Trunc(60*Frac(mySpaRecord.sunrise));
  a := 60*Frac(mySpaRecord.sunrise);
  sec := Trunc(60*Frac(a));
  Memo1.Lines.Add('Sunrise:       '+Int2StrPad0(hrs,2)+':'+Int2StrPad0(min,2)+':'+Int2StrPad0(sec,2)+' Local Time');

  hrs := Trunc(mySpaRecord.suntransit);
  min := Trunc(60*Frac(mySpaRecord.suntransit));
  a := 60*Frac(mySpaRecord.suntransit);
  sec := Trunc(60*Frac(a));
  Memo1.Lines.Add('Suntransit:    '+Int2StrPad0(hrs,2)+':'+Int2StrPad0(min,2)+':'+Int2StrPad0(sec,2)+' Local Time');

  hrs := Trunc(mySpaRecord.sunset);
  min := Trunc(60*Frac(mySpaRecord.sunset));
  a := 60*Frac(mySpaRecord.sunset);
  sec := Trunc(60*Frac(a));
  Memo1.Lines.Add('Sunset:        '+Int2StrPad0(hrs,2)+':'+Int2StrPad0(min,2)+':'+Int2StrPad0(sec,2)+' Local Time');

  Memo1.Lines.Add('');
  Memo1.Lines.Add('Pan angle from South:       '+FloatToStr(mySpaRecord.azimuth - 180)+' degrees');
  Memo1.Lines.Add('Tilt angle from horizontal: '+FloatToStr(90 - mySpaRecord.zenith)+' degrees');
end;

procedure TForm1.Clear1Click(Sender: TObject);
begin
  Memo1.Clear;
end;

procedure TForm1.Test1Click(Sender: TObject);    // Standard Test Case
begin

  Edit1.Text := IntToStr(2003);
  Edit2.Text := IntToStr(10);
  Edit3.Text := IntToStr(17);
  Edit4.Text := IntToStr(12);
  Edit5.Text := IntToStr(30);
  Edit6.Text := IntToStr(30);
  Edit7.Text := FloatToStr(0);
  Edit8.Text := FloatToStr(67);
  Edit9.Text := FloatToStr(-7.0);
  Edit10.Text := FloatToStr(-105.1786);
  Edit11.Text := FloatToStr(39.742476);
  Edit12.Text := FloatToStr(1830.14);
  Edit13.Text := FloatToStr(820);
  Edit14.Text := FloatToStr(11);
  Edit15.Text := FloatToStr(30);
  Edit16.Text := FloatToStr(-10);
  Edit17.Text := FloatToStr(0.5667);      

  mySpaRecord.year := StrToInt(Edit1.Text);
  mySpaRecord.month := StrToInt(Edit2.Text);
  mySpaRecord.day := StrToInt(Edit3.Text);
  mySpaRecord.hour := StrToInt(Edit4.Text);
  mySpaRecord.minute := StrToInt(Edit5.Text);
  mySpaRecord.second := StrToInt(Edit6.Text);
  mySpaRecord.delta_ut1 := StrToFloat(Edit7.Text);
  mySpaRecord.delta_t := StrToFloat(Edit8.Text);
  mySpaRecord.timezone := StrToFloat(Edit9.Text);
  mySpaRecord.longitude := StrToFloat(Edit10.Text);
  mySpaRecord.latitude := StrToFloat(Edit11.Text);
  mySpaRecord.elevation := StrToFloat(Edit12.Text);
  mySpaRecord.pressure := StrToFloat(Edit13.Text);
  mySpaRecord.temperature := StrToFloat(Edit14.Text);
  mySpaRecord.slope := StrToFloat(Edit15.Text);
  mySpaRecord.azm_rotation := StrToFloat(Edit16.Text);
  mySpaRecord.atmos_refract := StrToFloat(Edit17.Text);

  mySpaRecord.switch := SPA_ALL;

  mySpaRecordPtr := @mySpaRecord;

end;

procedure TForm1.Whitby1Click(Sender: TObject);  // Coordinates for Whitby, ON
begin
  Edit1.Text := IntToStr(2015);
  Edit2.Text := IntToStr(3);
  Edit3.Text := IntToStr(11);
  Edit4.Text := IntToStr(18);
  Edit5.Text := IntToStr(0);
  Edit6.Text := IntToStr(0);
  Edit7.Text := FloatToStr(0);
  Edit8.Text := FloatToStr(67);
  Edit9.Text := FloatToStr(-4.0);
  Edit10.Text := FloatToStr(-78.960161);
  Edit11.Text := FloatToStr(43.906411);
  Edit12.Text := FloatToStr(110);
  Edit13.Text := FloatToStr(1013);
  Edit14.Text := FloatToStr(4);
  Edit15.Text := FloatToStr(0);
  Edit16.Text := FloatToStr(0);
  Edit17.Text := FloatToStr(0.5667);

  mySpaRecord.year := StrToInt(Edit1.Text);
  mySpaRecord.month := StrToInt(Edit2.Text);
  mySpaRecord.day := StrToInt(Edit3.Text);
  mySpaRecord.hour := StrToInt(Edit4.Text);
  mySpaRecord.minute := StrToInt(Edit5.Text);
  mySpaRecord.second := StrToInt(Edit6.Text);
  mySpaRecord.delta_ut1 := StrToFloat(Edit7.Text);
  mySpaRecord.delta_t := StrToFloat(Edit8.Text);
  mySpaRecord.timezone := StrToFloat(Edit9.Text);
  mySpaRecord.longitude := StrToFloat(Edit10.Text);
  mySpaRecord.latitude := StrToFloat(Edit11.Text);
  mySpaRecord.elevation := StrToFloat(Edit12.Text);
  mySpaRecord.pressure := StrToFloat(Edit13.Text);
  mySpaRecord.temperature := StrToFloat(Edit14.Text);
  mySpaRecord.slope := StrToFloat(Edit15.Text);
  mySpaRecord.azm_rotation := StrToFloat(Edit16.Text);
  mySpaRecord.atmos_refract := StrToFloat(Edit17.Text);

  mySpaRecord.switch := SPA_ALL;

  mySpaRecordPtr := @mySpaRecord;
end;

procedure TForm1.Button2Click(Sender: TObject); {Whitby All - for spreadsheet}
var
  Date: string;
  Month: string;
  Day: string;
  Year: string;
  Time: string;
  Hour: string;
  Minute: string;
  Second: string;

  i: integer;
begin
  Whitby1Click(self);

  Date := DateToStr(Now);
  Day := GetFirstToken(Date,'/');
  Date := StripFirstToken(Date,'/');
  Month := GetFirstToken(Date,'/');
  Year := StripFirstToken(Date,'/');

  Time := TimeToStr(Now);
  Hour := GetFirstToken(Time,':');
  Time := StripFirstToken(Time,':');
  Minute := GetFirstToken(Time,':');
  Time := StripFirstToken(Time,':');
  Second := GetFirstToken(Time,' ');

  Edit1.Text := Year;
  Edit2.Text := Month;
  Edit3.Text := Day;
  Edit4.Text := Hour;
  Edit5.Text := Minute;
  Edit6.Text := Second;

  mySpaRecord.year := StrToInt(Edit1.Text);
  mySpaRecord.month := StrToInt(Edit2.Text);
  mySpaRecord.day := StrToInt(Edit3.Text);
  mySpaRecord.hour := StrToInt(Edit4.Text);
  mySpaRecord.minute := StrToInt(Edit5.Text);
  mySpaRecord.second := StrToInt(Edit6.Text);

  mySpaRecord.switch := SPA_ALL;

  mySpaRecordPtr := @mySpaRecord;

  Spa_Calculate(mySpaRecord);

  Memo1.Clear;
  Memo1.Lines.Add('Pan values -East +West of due South');
  for i := 7 to 20 do
  begin
    Edit4.Text := IntToStr(i);
    Edit5.Text := IntToStr(0);
    Edit6.Text := IntToStr(0);
    mySpaRecord.hour := StrToInt(Edit4.Text);
    mySpaRecord.minute := StrToInt(Edit5.Text);
    mySpaRecord.second := StrToInt(Edit6.Text);

    Spa_Calculate(mySpaRecord);

    Memo1.Lines.Add(FloatToStr(mySpaRecord.azimuth - 180));
  end;

  Memo1.Lines.Add('');
  Memo1.Lines.Add('Tilt values from horizontal');
  for i := 7 to 20 do
  begin
    Edit4.Text := IntToStr(i);
    Edit5.Text := IntToStr(0);
    Edit6.Text := IntToStr(0);
    mySpaRecord.hour := StrToInt(Edit4.Text);
    mySpaRecord.minute := StrToInt(Edit5.Text);
    mySpaRecord.second := StrToInt(Edit6.Text);

    Spa_Calculate(mySpaRecord);

    Memo1.Lines.Add(FloatToStr(90 - mySpaRecord.zenith));
  end;
end;

procedure TForm1.Button3Click(Sender: TObject);  {Whitby Now - using PC dat and time}
var
  Date: string;
  Month: string;
  Day: string;
  Year: string;
  Time: string;
  Hour: string;
  Minute: string;
  Second: string;
  AmPm: string;

  hrs: integer;
  min: integer;
  sec: integer;
  a: double;
  CompResult: integer;

  Present: TDateTime;
  Nhrs: word;
  Nmin: word;
  Nsec: word;
  MSec: word;
begin

  Whitby1Click(self);

  Date := DateToStr(Now);
  Day := GetFirstToken(Date,'/');
  Date := StripFirstToken(Date,'/');
  Month := GetFirstToken(Date,'/');
  Year := StripFirstToken(Date,'/');

  Time := TimeToStr(Now);
  Hour := GetFirstToken(Time,':');
  Time := StripFirstToken(Time,':');
  Minute := GetFirstToken(Time,':');
  Time := StripFirstToken(Time,':');
  Second := GetFirstToken(Time,' ');
  Time := StripFirstToken(Time,' ');
  AmPm := Time;

  Edit1.Text := Year;
  Edit2.Text := Month;
  Edit3.Text := Day;

  CompResult := StrComp(PChar(AmPm),PChar('PM'));
  Memo1.Lines.Add(TimeToStr(Now));
  //Memo1.Lines.Add(AmPm);
  if ((CompResult = 0) AND (StrToInt(Hour) <> 12)) then Hour := IntToStr(StrToInt(Hour) + 12);


  Edit4.Text := Hour;
  Edit5.Text := Minute;
  Edit6.Text := Second;

  mySpaRecord.year := StrToInt(Edit1.Text);
  mySpaRecord.month := StrToInt(Edit2.Text);
  mySpaRecord.day := StrToInt(Edit3.Text);
  mySpaRecord.hour := StrToInt(Edit4.Text);
  mySpaRecord.minute := StrToInt(Edit5.Text);
  mySpaRecord.second := StrToInt(Edit6.Text);

  mySpaRecord.switch := SPA_ALL;

  mySpaRecordPtr := @mySpaRecord;

  Spa_Calculate(mySpaRecord);

  Memo1.Clear;
  Memo1.Lines.Add('Zenith:        '+FloatToStr(mySpaRecord.zenith));
  //Memo1.Lines.Add('Azimuth astro: '+FloatToStr(mySpaRecord.azimuth_astro));
  Memo1.Lines.Add('Azimuth:       '+FloatToStr(mySpaRecord.azimuth));
  //Memo1.Lines.Add('Incidence:     '+FloatToStr(mySpaRecord.incidence));

  
  Memo1.Lines.Add('');
  hrs := Trunc(mySpaRecord.sunrise);
  min := Trunc(60*Frac(mySpaRecord.sunrise));
  a := 60*Frac(mySpaRecord.sunrise);
  sec := Trunc(60*Frac(a));
  Memo1.Lines.Add('Sunrise:       '+Int2StrPad0(hrs,2)+':'+Int2StrPad0(min,2)+':'+Int2StrPad0(sec,2));

  hrs := Trunc(mySpaRecord.suntransit);
  min := Trunc(60*Frac(mySpaRecord.suntransit));
  a := 60*Frac(mySpaRecord.suntransit);
  sec := Trunc(60*Frac(a));
  Memo1.Lines.Add('Suntransit:    '+Int2StrPad0(hrs,2)+':'+Int2StrPad0(min,2)+':'+Int2StrPad0(sec,2));

  hrs := Trunc(mySpaRecord.sunset);
  min := Trunc(60*Frac(mySpaRecord.sunset));
  a := 60*Frac(mySpaRecord.sunset);
  sec := Trunc(60*Frac(a));
  Memo1.Lines.Add('Sunset:        '+Int2StrPad0(hrs,2)+':'+Int2StrPad0(min,2)+':'+Int2StrPad0(sec,2));

  Memo1.Lines.Add('');
  Memo1.Lines.Add('Pan angle from South:       '+FloatToStr(mySpaRecord.azimuth - 180));
  Memo1.Lines.Add('Tilt angle from horizontal: '+FloatToStr(90 - mySpaRecord.zenith));

end;

procedure TForm1.Button4Click(Sender: TObject);  // Test for function ArcTan2(const Y, X: Extended): Extended;
var
  Temp1: double;
begin
  Temp1 := ArcTan2(1.0,2.0);
  Memo1.Lines.Add(FloatToStr(Temp1));
end;

end.
